library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

library work;
use work.axi4_lite_register;

entity testbench is
generic
(
  C_S_AXI_DATA_WIDTH             : integer              := 32;
  C_S_AXI_ADDR_WIDTH             : integer              := 32;
  C_S_AXI_MIN_SIZE               : std_logic_vector     := X"000001FF";
  C_USE_WSTRB                    : integer              := 0;
  C_DPHASE_TIMEOUT               : integer              := 8;
  C_BASEADDR                     : std_logic_vector     := X"FFFFFFFF";
  C_HIGHADDR                     : std_logic_vector     := X"00000000";
  C_FAMILY                       : string               := "virtex6";
  C_NUM_REG                      : integer              := 32;
  C_SLV_DWIDTH                   : integer              := 32
);

end testbench;

architecture Behavioral of testbench is
    signal D_IN					          : std_logic_vector(7 downto 0);
    signal D_OUT					      : std_logic_vector(7 downto 0);
    signal InteruptOut				      : std_logic; 

    signal S_AXI_ACLK                     :  std_logic;
    signal S_AXI_ARESETN                  :  std_logic;
    signal S_AXI_AWADDR                   :  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
    signal S_AXI_AWVALID                  :  std_logic;
    signal S_AXI_WDATA                    :  std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
    signal S_AXI_WSTRB                    :  std_logic_vector((C_S_AXI_DATA_WIDTH/8)-1 downto 0);
    signal S_AXI_WVALID                   :  std_logic;
    signal S_AXI_BREADY                   :  std_logic;
    signal S_AXI_ARADDR                   :  std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
    signal S_AXI_ARVALID                  :  std_logic;
    signal S_AXI_RREADY                   :  std_logic;
    signal S_AXI_ARREADY                  : std_logic;
    signal S_AXI_RDATA                    : std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
    signal S_AXI_RRESP                    : std_logic_vector(1 downto 0);
    signal S_AXI_RVALID                   : std_logic;
    signal S_AXI_WREADY                   : std_logic;
    signal S_AXI_BRESP                    : std_logic_vector(1 downto 0);
    signal S_AXI_BVALID                   : std_logic;
    signal S_AXI_AWREADY                  : std_logic;

	Constant ClockPeriod : TIME := 5 ns;
	Constant ClockPeriod2 : TIME := 10 ns;
	shared variable ClockCount : integer range 0 to 50_000 := 10;
	signal sendIt : std_logic := '0';
	signal readIt : std_logic := '0';
begin


AXI4_LITE_REGISTER_I : entity work.axi4_lite_register
  generic map
  (
    C_S_AXI_DATA_WIDTH             => C_S_AXI_DATA_WIDTH,
    C_S_AXI_ADDR_WIDTH             => C_S_AXI_ADDR_WIDTH,
    C_S_AXI_MIN_SIZE               => C_S_AXI_MIN_SIZE,
    C_USE_WSTRB                    => C_USE_WSTRB,
    C_DPHASE_TIMEOUT               => C_DPHASE_TIMEOUT,
    C_BASEADDR                     => C_BASEADDR,
    C_HIGHADDR                     => C_HIGHADDR,
    C_FAMILY                       => C_FAMILY,
    C_NUM_REG                      => C_NUM_REG,
    C_SLV_DWIDTH                   => C_SLV_DWIDTH  
  )
  port map
  (
    D_IN					       => D_IN,
    D_OUT					       => D_OUT,
    InteruptOut				       => InteruptOut,
    
    S_AXI_ACLK                     => S_AXI_ACLK,
    S_AXI_ARESETN                  => S_AXI_ARESETN,
    S_AXI_AWADDR                   => S_AXI_AWADDR,
    S_AXI_AWVALID                  => S_AXI_AWVALID,
    S_AXI_WDATA                    => S_AXI_WDATA,
    S_AXI_WSTRB                    => S_AXI_WSTRB,
    S_AXI_WVALID                   => S_AXI_WVALID,
    S_AXI_BREADY                   => S_AXI_BREADY,
    S_AXI_ARADDR                   => S_AXI_ARADDR,
    S_AXI_ARVALID                  => S_AXI_ARVALID,
    S_AXI_RREADY                   => S_AXI_RREADY,
    S_AXI_ARREADY                  => S_AXI_ARREADY,
    S_AXI_RDATA                    => S_AXI_RDATA,
    S_AXI_RRESP                    => S_AXI_RRESP,
    S_AXI_RVALID                   => S_AXI_RVALID,
    S_AXI_WREADY                   => S_AXI_WREADY,
    S_AXI_BRESP                    => S_AXI_BRESP,
    S_AXI_BVALID                   => S_AXI_BVALID,
    S_AXI_AWREADY                  => S_AXI_AWREADY
  );
  
 GENERATE_REFCLOCK : process
 begin
   wait for (ClockPeriod / 2);
   ClockCount:= ClockCount+1;
   S_AXI_ACLK <= '1';
-- DATA<=to_vector(C_S_AXI_DATA_WIDTH,ClockCount);--x"AAAAAAAA11111111";
   wait for (ClockPeriod / 2);
   S_AXI_ACLK <= '0';
 end process;

 send : PROCESS
 BEGIN
    S_AXI_AWVALID<='0';
    S_AXI_WVALID<='0';
    S_AXI_BREADY<='0';
    loop
        wait until sendIt = '1';
        wait until S_AXI_ACLK= '0';
            S_AXI_AWVALID<='1';
            S_AXI_WVALID<='1';
        wait until (S_AXI_AWREADY and S_AXI_WREADY) = '1';  --Client ready to read address/data        
            S_AXI_BREADY<='1';
        wait until S_AXI_BVALID = '1';  -- Write result valid
            assert S_AXI_BRESP = "00" report "AXI data not written" severity failure;
            S_AXI_AWVALID<='0';
            S_AXI_WVALID<='0';
            S_AXI_BREADY<='1';
        wait until S_AXI_BVALID = '0';  -- All finished
            S_AXI_BREADY<='0';
    end loop;
 END PROCESS send;
 
  read : PROCESS
  BEGIN
    S_AXI_ARVALID<='0';
    S_AXI_RREADY<='0';
     loop
         wait until readIt = '1';
         wait until S_AXI_ACLK= '0';
             S_AXI_ARVALID<='1';
             S_AXI_RREADY<='1';
         wait until (S_AXI_RVALID and S_AXI_ARREADY) = '1';  --Client provided data
            assert S_AXI_RRESP = "00" report "AXI data not written" severity failure;
             S_AXI_ARVALID<='0';
            S_AXI_RREADY<='0';
     end loop;
  END PROCESS read;

 tb : PROCESS
 BEGIN
        D_IN <=x"0F";
        S_AXI_ARESETN<='0';
        sendIt<='0';
    wait for 5 ns;
        S_AXI_ARESETN<='1';

        S_AXI_AWADDR<=x"00000000";
        S_AXI_WDATA<=x"00000000";
        sendIt<='1';                --Start AXI Write to Slave
    wait until S_AXI_BVALID = '0';  --AXI Write finished 
        sendIt<='0';wait for 1 ns;  --Clear Start Flag
        S_AXI_AWADDR<=x"00000000";
        S_AXI_WDATA<=x"00000001";
        sendIt<='1';
    wait until S_AXI_BVALID = '0';
        sendIt<='0';wait for 1 ns;  --Clear Start Flag
        S_AXI_AWADDR<=x"00000000";
        S_AXI_WDATA<=x"00000002";
        sendIt<='1';
    wait until S_AXI_BVALID = '0';
        sendIt<='0';wait for 1 ns;  --Clear Start Flag
        S_AXI_AWADDR<=x"00000004";
        S_AXI_WDATA<=x"A5A5A5A5";
        sendIt<='1';
    wait until S_AXI_BVALID = '0';   
        S_AXI_ARADDR<=x"00000000";
        readIt<='1';
    wait until S_AXI_ARREADY = '0';
        readIt<='0';wait for 1 ns;
        S_AXI_ARADDR<=x"00000004";
        readIt<='1';
            
     wait; -- will wait forever
 END PROCESS tb;

end Behavioral;
